/* 
 * Copyright (c) 2002 LAAS/CNRS                       --  Wed Jun 12 2002
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 *
 * $Id: imuStruct.h,v 1.3 2007/06/29 15:11:50 tpeynot Exp $
 */
#ifndef H_IMUSTRUCT
#define H_IMUSTRUCT



/****** Constants **************************/
#define IMU_MAX_STRLEN		64

#define IMU_GRAVITY_ACCEL  9.81

/* Initialization values for Integrated Data */
#define IMU_DEFAULT_INIT_YAW (0.0)
#define IMU_DEFAULT_INIT_PITCH (0.0)
#define IMU_DEFAULT_INIT_ROLL (0.0)

#define IMU_DEFAULT_INIT_XSPEED (0.0)
#define IMU_DEFAULT_INIT_YSPEED (0.0)
#define IMU_DEFAULT_INIT_ZSPEED (0.0)

#define IMU_DEFAULT_INIT_XPOS (0.0)
#define IMU_DEFAULT_INIT_YPOS (0.0)
#define IMU_DEFAULT_INIT_ZPOS (0.0)

#define IMU_DEFAULT_XACCEL_BIAS (0.0)
#define IMU_DEFAULT_YACCEL_BIAS (0.0)
#define IMU_DEFAULT_ZACCEL_BIAS (0.0)

#define IMU_DEFAULT_YAWRATE_BIAS (0.0)
#define IMU_DEFAULT_PITCHRATE_BIAS (0.0)
#define IMU_DEFAULT_ROLLRATE_BIAS (0.0)
/******************************************/




typedef struct ImuDevice {
   char port[IMU_MAX_STRLEN];
   int baud;
   int period;

#ifdef ATRV
#define IMU_DEFAULT_PORT	"/dev/ttyR3"
#define IMU_DEFAULT_BAUD	38400
#define IMU_DEFAULT_PERIOD	2
#elif defined(ATRV_JUNIOR)   /* TODO: put the right values for Gromit */
#define IMU_DEFAULT_PORT	"/dev/ttyR4"
#define IMU_DEFAULT_BAUD	38400
#define IMU_DEFAULT_PERIOD	2
#elif defined(B21R)
#define IMU_DEFAULT_PORT	"/dev/ttyR1"
#define IMU_DEFAULT_BAUD	38400
#define IMU_DEFAULT_PERIOD      1
#else
#error "No Robot defined"
#endif

} ImuDevice;

 
#define IMU_POSTER_NAME		"imuSensors"


typedef struct ImuPoster {
   double yaw;			/* integrated angles */
   double pitch;
   double roll;

   double yawRate;		/* angular velocities */
   double pitchRate;
   double rollRate;

   double yawRateBias;		/* Biases on angular velocities */
   double pitchRateBias;
   double rollRateBias;

   double yawRateNoBias;       /* angular velocities after bias compensation */
   double pitchRateNoBias;
   double rollRateNoBias;

  /********************/
  double yawVariance;
  double pitchVariance;
  double rollVariance;
  /********************/

   double xAccel;		/* "raw" accelerations */
   double yAccel;
   double zAccel;

  double xAccelBias;  /* Biases on accelerations to be compensated */
  double yAccelBias;
  double zAccelBias;

  double xAccelNoBias;  /* Accels after bias compensation */
  double yAccelNoBias;
  double zAccelNoBias;

  /***** Thierry ************************/
  double xAccelFiltered;		/* accelerations filtered */
  double yAccelFiltered;
  double zAccelFiltered;

  double yawRateFiltered;		/* angular velocities filtered*/
  double pitchRateFiltered;
  double rollRateFiltered;

  double xSpeed;                        /* speed x (m/s) */
  double ySpeed;                        /* speed y (m/s) */
  double zSpeed;                        /* speed z (m/s) */

  double xSpeedVariance;                /* Variances for x-y-zSpeed */
  double ySpeedVariance;
  double zSpeedVariance;

  /* Positions in the Robot Frame */
  /* Peut-etre que ce serait mieux de mettre les incrments de position */
  /*  pour calculer les positions dans un referentiel + global + tard */
  double xPos;
  double yPos;
  double zPos;
  double xPosVariance;
  double yPosVariance;
  double zPosVariance;

  /*** Attitude angles as in LAMA classical frame ***/
  /*** These Name are temporary ***/
  double phiLama;
  double psiLama;

  /***** Th End *************************/

} ImuPoster;



/*---- Ajouts Thierry ----------------------------------------*/

#define IMU_DUMPPERIOD_MIN 1
#define IMU_DUMP_FILENAME_MAX (64)
#define IMU_DEFAULT_DUMPFILENAME "/home/tpeynot/robots/atrv/modules/imu/IMUdump.data"

typedef struct IMU_DUMPONFILE_PARAMS
{
  int     dumpPeriod;   /* Nb de cycles de la tache d'exec entre chaque dump */
  int     unused;
  char    fileName[IMU_DUMP_FILENAME_MAX];  /* Name of the file where data is dumped */

} IMU_DUMPONFILE_PARAMS;



/* Initial Values of Integrated Data */
/* Input of Request InitIntegratedData */
typedef struct IMU_INTEGRATEDDATA_INITVALUES
{
  double  xSpeedInitValue;
  double  ySpeedInitValue;
  double  zSpeedInitValue;

  double  xPosInitValue;
  double  yPosInitValue;
  double  zPosInitValue;

  double  rollInitValue;
  double  pitchInitValue;
  double  yawInitValue;

} IMU_INTEGRATEDDATA_INITVALUES;



/*---- Default Values for Variances ----*/
#define IMU_DEFAULT_XACCEL_VARIANCE 0.0
#define IMU_DEFAULT_YACCEL_VARIANCE 0.0
#define IMU_DEFAULT_ZACCEL_VARIANCE 0.0

#define IMU_DEFAULT_YAWRATE_VARIANCE 0.0
#define IMU_DEFAULT_PITCHRATE_VARIANCE 0.0
#define IMU_DEFAULT_ROLLRATE_VARIANCE 0.0
/*--------------------------------------*/


/*** Separer Variances statiques et variances en mouvement? *********/

typedef struct IMU_VARIANCES
{
  double xAccelVariance;    /* Variances for the accelerations */
  double yAccelVariance; 
  double zAccelVariance; 
  
  double yawRateVariance;   /* Variances for angular speeds */
  double pitchRateVariance;
  double rollRateVariance;

} IMU_VARIANCES;





/********* Struct pour parametres du filtrage ************/

#define IMU_DEFAULT_NUMSAMPLES_FILTER 4
/** On pourrait par exemple mettre explicitement ici la periode (en nombre de ticks) de fonctionnement du module RFLEX (c'est justement 4 a priori pour le moemen) */

typedef enum IMU_FILTER_MODE {
	IMU_FILTER_ON,
	IMU_FILTER_OFF
} IMU_FILTER_MODE;

typedef struct IMU_FILTER_PARAMS
{
  IMU_FILTER_MODE angularSpeedsFilterMode;  /* Flag Filter ON/OFF */
  int numSamplesUsedToFilter_angSpeeds;

  IMU_FILTER_MODE accelFilterMode;  /* Flag Filter ON/OFF */
  int numSamplesUsedToFilter_accel;

} IMU_FILTER_PARAMS;




/********* Params for request GetSensorsData ************/

#define IMU_DEFAULT_BASEFILENAME "/home/tpeynot/robots/atrv/modules/imu/imuSensorsData."

typedef struct IMU_GETSENSORSDATA_PARAMS
{
  char    baseFileName[IMU_DUMP_FILENAME_MAX];

} IMU_GETSENSORSDATA_PARAMS;


/********* Params for request SetSensorsDataNumFile ************/

typedef struct IMU_SETSENSORSDATA_PARAMS
{
  int getSensorsData_numFile;  /* ID number of the next file generated by the request GetSensorsData */
  int unused;
} IMU_SETSENSORSDATA_PARAMS;


/********* Params for request SetAccelBiases ************/

typedef struct IMU_SETACCELBIASES_PARAMS
{
  double xAccelBias;
  double yAccelBias;
  double zAccelBias;

} IMU_SETACCELBIASES_PARAMS;


/********* Params for request ComputeAccelBiases ************/

#define IMU_DEFAULT_COMPUTEBIAS_DURATION 10.0
/* Default duration (in s) for the calculation of the accel averages */


typedef struct IMU_COMPUTEACCELBIASES_PARAMS
{
  double phiMeasured; /* Attitude angles measured by the operator (in rad) */
  double psiMeasured;

  double duration;   /* Time for calculation of averages */

} IMU_COMPUTEACCELBIASES_PARAMS;



/********* Params for request ComputeAngspeedsBiases ************/

typedef struct IMU_COMPUTEANGSPEEDSBIASES_PARAMS
{
  double duration;   /* Time for calculation of averages */

} IMU_COMPUTEANGSPEEDSBIASES_PARAMS;



/********* Params for the Backstepping Attitude Estimation ******/

#define IMU_DEFAULT_BACKSTEPPING_KmGAIN 1.0

#define IMU_DEFAULT_BACKSTEPPING_KbGAIN 0.1

typedef enum IMU_BACKSTEP_MODE {
	IMU_BACKSTEP_ON,
	IMU_BACKSTEP_OFF
} IMU_BACKSTEP_MODE;

typedef struct IMU_BACKSTEPPING_PARAMS
{
  IMU_BACKSTEP_MODE mode;

  double Km;  /* Gain  */

  double Kb;  /* Gain */

} IMU_BACKSTEPPING_PARAMS;



/* Results of attitude estimation using backstepping technique */
typedef struct IMU_ATTITUDE_ESTIMATION
{
  double gravityVector[3];  /* Estimated gravity vector */
  
  double Bestimated[3];    /* Estimated Angular Speeds biases */

  double attitudeAngles[2];  /* Estimated attitude angles */
  /* attitudeAngles[0]=pitch, attitudeAngles[1]=roll */


} IMU_ATTITUDE_ESTIMATION;




/***********************************************************************/
/**** Structures for the FlatTerrain Monitor ******/

/***********************************************************************/
/* Struct containing data from the config monitor */

typedef enum IMU_MONITOR_STATUS {
  IMU_MONITOR_OFF, IMU_MONITOR_ON
} IMU_MONITOR_STATUS;

typedef struct IMU_MONITOR_DATA {
  float pitchRateEnergy;
  float rollRateEnergy;

  float probaPitch;  /* "shaped" distPitch */
  float probaRoll;  /* "shaped" distRoll  */

  float behavior;  /* Final "behavior pseudo-proba"  */

  float time;  /* Time associated to IMU data */

  IMU_MONITOR_STATUS  status; /* Status flag to know if Monitor activated or not */
  int  unused;

} IMU_MONITOR_DATA;



/***********************************************************************/
/* Struct containing params for the FlatTerrainMonitor request */

typedef enum IMU_MONITOR_LOG_FLAG {
  IMU_MONITOR_LOG_OFF, IMU_MONITOR_LOG_ON
} IMU_MONITOR_LOG_FLAG;

#define IMU_MONITOR_LOG_DEFAULT_FILENAME	"/home/tpeynot/Images/imuMonitorLog.data"

#define IMU_MONITOR_MAXFLAT_DEFAULT (0.001)
#define IMU_MONITOR_MINROUGH_DEFAULT (0.01)
#define IMU_MONITOR_SIGMASIGMOID_DEFAULT (0.004)

typedef struct IMU_MONITOR_PARAMS {
  int  timewidth_nbCycles;      /* width of time window for energy computation (in number of MonitorTask cycles) */
  IMU_MONITOR_LOG_FLAG logFlag;
  char fileName[IMU_DUMP_FILENAME_MAX];  /* Name of File where to save the logs */
  
  /* Parameters concerning the "shaping" to get the pseudo-probas from the energies */
  float   MaxFlat;   /* highest level of Energy correspond. to Flat Terrain */
  float   MinRough;  /* Lowest level of Energy to get a Rough Terrain */
  float   sigmaSigmoid;  /* Sigma for the sigmoid */
  int     unused;

} IMU_MONITOR_PARAMS;





/*------------------------------------------------------------*/

#endif /* H_IMUSTRUCT */
